<?php
/**
 * $Id: slideshow_creator.cck.inc,v 1.4.2.1 2008/03/25 00:27:50 brmassa Exp $
 * @file slideshow_creator.cck.inc
 * @author Bruno Massa http://drupal.org/user/67164
 * Create true slideshows using CCK module.
 * Project Page  : http://drupal.org/project/slideshow_creator
 * Support Queue : http://drupal.org/project/issues/slideshow_creator
 */

/**
 * Implementation of hook_field_info().
 *
 * Its a CCK hook that describes a type of fields. On CCK taxonomy,
 * fields are the types of data (like text, numbers or slideshows),
 * that can be used by several widgets to store and several formatters
 * display this data.
 */
function slideshow_creator_field_info() {
  return array(
    'slideshow' => array('label' => 'Slideshow'),
  );
}

/**
 * Implementation of hook_field_formatter().
 */
function slideshow_creator_field_formatter($field, $item, $formatter) {
  $attributes = array();

  // Add special attributes to this slideshow
  array_push($attributes, empty($item['name'])   ? '' : 'name='.   $item['name']);
  array_push($attributes, empty($item['rotate']) ? '' : 'rotate='. $item['rotate']);
  array_push($attributes, empty($item['blend'])  ? '' : 'blend='.  $item['blend']);
  array_push($attributes, empty($item['height']) ? '' : 'height='. $item['height']);
  array_push($attributes, empty($item['width'])  ? '' : 'width='.  $item['width']);
  array_push($attributes, empty($item['layout']) ? '' : 'layout='. $item['layout']);

  // Check it there is images on the slideshow: it can be
  //   - direct list of images
  //   - local directory where several images are
  if (!empty($item['url']) or !empty($item['dir'])) {
    if ($urls = split("\n", $item['url'])) {
      while (list($index, $url) = each($urls)) {
        $urls[$index] = 'img=|'. $url;
      }
    }
    if ($dirs = split("\n", $item['dir'])) {
      while (list($index, $url) = each($dirs)) {
        $dirs[$index] = 'dir=|'. $url;
      }
    }
    $images = array_merge($urls, $dirs);

    // Include the proper file to deal with this
    include_once drupal_get_path('module', 'slideshow_creator') .'/slideshow_creator.inc';

    // Build the Slideshow Creator tag and send it to
    // the parser, that will transform it into a slideshow
    return _slideshow_creator_process_text('[slideshow: 2,'. implode(',', $attributes) .','. implode(',', $images) .']');
  }
  return '';
}

/**
 * Implementation of hook_field_formatter_info().
 *
 * Its a CCK hook, that describes each formatter. No CCK taxonomy,
 * formatters displays a data saved on "fields" on screen.
 */
function slideshow_creator_field_formatter_info() {
  $formatters = array(
    'default' => array(
      'label' => 'Default',
      'field types' => array('slideshow'),
    ),
  );
  return $formatters;
}

/**
 * Implementation of hook_field_settings(): how this field should behave
 */
function slideshow_creator_field_settings($op, $field) {
  switch ($op) {
    case 'database columns':
      return array(
        'ssc'     => array('type' => 'text'),
      );
  }
}

/**
 * Implementation of hook_widget().
 *
 * defines the widget fields
 */
function slideshow_creator_widget(&$form, &$form_state, $field, $items, $delta = 0) {
  $element['ssc'] = array(
    '#type'          => 'fieldset',
    '#title'         => t($field['widget']['label']),
    '#weight'        => $field['widget']['weight'],
    '#collapsible'   => TRUE,
    '#collapsed'     => FALSE,
    '#tree'          => TRUE
  );
  $element['ssc']['url'] = array(
    '#type'          => 'textarea',
    '#title'         => t('Image'),
    '#default_value' => $items['ssc']['url'],
    '#description'   => t('On each line, fill your slides using the <b>IMAGE_URL|LINK|TITLE|DESCRIPTION|TARGET</b> pattern.')
  );
  $element['ssc']['dir'] = array(
    '#type'          => 'textarea',
    '#title'         => t('Image directory'),
    '#default_value' => $items['ssc']['dir'],
    '#description'   => t('On each line, fill your slides using the <b>DIR_IMAGE|DIR_RECURSIVE|DIR_LINK|TITLE|DESCRIPTION|TARGET</b> pattern.')
  );
  $element['ssc']['rotate'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Rotation'),
    '#default_value' => $items['ssc']['rotate'],
    '#description'   => t('The rotation speed, in seconds. Leave it blank for no rotation.'),
    '#size'          => 5,
    '#weight'        => 1
  );
  $element['ssc']['blend'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Blend'),
    '#default_value' => $items['ssc']['blend'],
    '#description'   => t('The blending speed, in seconds. Leave it blank for no blending.'),
    '#size'          => 5,
    '#weight'        => 1
  );
  $element['ssc']['width'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Width'),
    '#default_value' => $items['ssc']['width'],
    '#description'   => t('The image width, in pixels.'),
    '#size'          => 5,
    '#weight'        => 2
  );
  $element['ssc']['height'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Height'),
    '#default_value' => $items['ssc']['height'],
    '#description'   => t('The image height, in pixels.'),
    '#size'          => 5,
    '#weight'        => 2
  );
  $element['ssc']['layout'] = array(
    '#type'          => 'select',
    '#title'         => t('Layout'),
    '#default_value' => $items['ssc']['layout'],
    '#description'   => t('The customized layout name. Leave it blank for default layout.'),
    '#options'       => array(
      'default' => t('Default'),
      'reverse' => t('Reverse'),
      'bottom'  => t('Bottom'),
      'top'     => t('Top'),
    ),
    '#weight'        => 3
  );
  $element['ssc']['name'] = array(
    '#type'          => 'value',
    '#value'         => ' ',
  );
  return $element;
}

/**
 * Implementation of hook_widget_info().
 *
 * Its a CCK hook, that describes each widget. No CCK taxonomy,
 * widgets are responsible for getting the information from the
 * user and store it on a standard form, a CCK "field".
 */
function slideshow_creator_widget_info() {
  return array(
    'slideshow' => array(
      'label'       => t('Slideshow'),
      'field types' => array('slideshow'),
    ),
  );
}

/**
 * Format the slideshow widget
 *
 * @ingroup themeable
 */
function theme_slideshow_creator_view_image($file, $alt = '', $title = '', $attributes = NULL, $getsize = TRUE) {
  return theme('slideshow_creator_image', $file, $alt, $title, $attributes , $getsize);
}
