<?php
/**
 * $Id: slideshow_creator.inc,v 1.1.2.1 2008/03/25 00:27:50 brmassa Exp $
 * @author Bruno Massa http://drupal.org/user/67164
 * @file slideshow_creator.inc
 * Create true slideshows using any image over internet with many other features.
 */

/**
 * Get all images on a given directory
 *
 * @param $path
 *  String. The absolute path to start the scanner
 * @param $path
 *  Boolean. TRUE for recursive scanning
 * @return
 *  Array. All images' paths
 */
function _slideshow_creator_explore_dir($path = '', $recursive = FALSE) {
  // A list of image extensions. It should be on lower and
  // upper cases to work on non GNU systems, like Solaris and Windows
  $exts = array('png', 'gif', 'jpeg', 'jpg', 'bmp',
    'PNG', 'GIF', 'JPEG', 'JPG', 'BMP');

  // Get all files from a given path
  $files = array();
  foreach ($exts as $ext) {
    $files = array_merge($files, glob("$path*.$ext"));
  }

  // If its a recursive scan, enter in all directories
  // and scan for more image files
  if ($dirs = glob("$path*", GLOB_ONLYDIR) and !empty($recursive)) {
    foreach ($dirs as $dir) {
      $files = array_merge($files, _slideshow_creator_explore_dir("$dir/"));
    }
  }

  return $files;
}

/**
 * The main user hook to display a slideshow.
 *
 * @param $output
 *   String. All the page text with the slideshow code
 * @return
 *   String. All the page text with the slideshow HTML/Javascript
 */
function _slideshow_creator_process_text($output) {
  // Search for Slideshow Creator tags on the text.
  if (preg_match_all('/\[slideshow:([^]]+)\]/i', $output, $matches, PREG_SET_ORDER)) {
    $max_ss = count($matches);

    // For each of the instances, replace the tag for
    // the real HTML code
    foreach ($matches as $match) {
      $attributes = preg_split('|(?<!\\\),|', $match[1]);
      if (!empty($attributes)) {
        $output = preg_replace('/\[slideshow:([^]]+)\]/i',
          _slideshow_creator_replace($max_ss, $attributes), $output, 1);
      }
    }
  }

  // Return the original text with all slideshows correctly
  // inserted
  return $output;
}

/**
 * The main user hook to display a slideshow.
 *
 * @param &$max_ss
 *   Number. How many slideshows there are on the page
 * @param &$attributes
 *   Array. Each slideshow attributes, like 'img', 'dir', 'rotate'
 * @return
 *   String, The HTML code for the slideshow
 */
function _slideshow_creator_replace(&$max_ss, &$attributes) {
  // To ensure that all slideshows have a unique
  // ID, the current_ss variable will hold a automatic
  // increasing number
  static $current_ss = 0;
  ++$current_ss;

  $output = '';

  // Using version system, its possible to expand the slideshow functionality
  // and support old versions
  switch ((int) $attributes[0]) {
    case 1:
    case 2:
    default:
      // Default settings for jQuery Cycle plugin
      $js = array(
        'pause'     => TRUE,
      );

      $slides = array(
        'total'   => 0,
        'layout'  => 'default',
      );

      $attributes_count = count($attributes);
      foreach ($attributes as $attribute) {
        $attribute = explode('=', $attribute);

        // If no , skip the validation
        if (empty($attribute[1])) {
          continue;
        }

        if (trim(strtolower($attribute[0])) == 'img') { // img tag: URL|link|title|description|target format
          $img_attributes = explode('|' , $attribute[1]);
          if (!empty($img_attributes[1])) {
            $slides['total']++;
            $slides[$slides['total']] = array(
              'src'         => _slideshow_creator_url(check_url(trim($img_attributes[1]))),
              'link'        => empty($img_attributes[2]) ? '': _slideshow_creator_url(url(trim($img_attributes[2]))),
              'title'       => empty($img_attributes[3]) ? '': check_plain(preg_replace('|\\\,|', ',', trim($img_attributes[3]))),
              'description' => empty($img_attributes[4]) ? '': check_plain(preg_replace('|\\\,|', ',', trim($img_attributes[4]))),
              'target'      => !empty($img_attributes[5]) ? check_plain(trim($img_attributes[5])) : '_blank',
            );
          }
        }
        elseif (trim(strtolower($attribute[0])) == 'dir') { // dir tag: scan all images from a given directory
          $dir_attributes = explode('|' , $attribute[1]);
          if (!empty($dir_attributes[1]) and $files = _slideshow_creator_explore_dir('./'. $dir_attributes[1], $dir_attributes[2])) {
            foreach ($files as $file) {
              $file = preg_replace('/^\.\//', '', $file);
              $slides['total']++;
              $slides[$slides['total']] = array(
                'src'         => $file,
                'link'        => empty($dir_attributes[3]) ? '': $file,
                'title'       => empty($dir_attributes[4]) ? '': check_plain(preg_replace('|\\\,|', ',', trim($dir_attributes[4]))),
                'description' => empty($dir_attributes[5]) ? '': check_plain(preg_replace('|\\\,|', ',', trim($dir_attributes[5]))),
                'target'      => !empty($dir_attributes[6]) ? check_plain(trim($dir_attributes[6])) : '_blank',
              );
            }
          }
        }
        else { // all other tags
          // For retro compatibility, we must filter some results
          // to simulate the previous behaviour
          if (trim($attribute[0]) == 'rotate') {
            $js['speed'] = check_plain(trim($attribute[1]));
          }
          elseif (trim($attribute[0]) == 'blend') {
          }
          else {
            $js[trim($attribute[0])] = check_plain(trim($attribute[1]));
          }
        }
      }

      // If at least one slide is valid, build the Slideshow
      if ($slides['total'] > 0) {
        // The proper JavaScript commands of next and previuos
        $js['next'] = '#ssc-next-'. $current_ss;
        $js['prev'] = '#ssc-previous-'. $current_ss;

        // To build the correct link for NEXT and PREVIOUS
        // buttons (for non-javascript browsers), we need to
        // know which slide each slideshow is now.
        if (!empty($_GET['ssc'])) {
          $status_sscs = explode(';', $_GET['ssc']);
          foreach ($status_sscs as $status_ssc) {
            $temp = explode(',' , $status_ssc);
            if ($temp[0] == $current_ss) {
              $sscs['next'][] = $temp[1] + 1 > $slides['total'] ? $current_ss .',1' : $current_ss .','. ($temp[1] + 1);
              $sscs['previous'][] = $temp[1] - 1 < 1 ? $current_ss .','. $slides['total'] : $current_ss .','. ($temp[1] - 1);
              $slides['current'] = $temp[1];
              $toggle = TRUE;
            }
            else {
              $sscs['next'][] = $status_ssc;
              $sscs['previous'][] = $status_ssc;
              $slides['current'] = 1;
            }
          }
        }
        if (empty($toggle)) {
          $sscs['next'][] = $slides['total'] > 1 ? check_plain($current_ss .',2') : check_plain($current_ss .',1');
          $sscs['previous'][] = check_plain($current_ss .','. $slides['total']);
          $slides['current'] = 1;
        }
        $links['next']      = check_plain(implode(';', $sscs['next'])) ;
        $links['previous']  = check_plain(implode(';', $sscs['previous'])) ;

        // The slideshow should be displayed using
        // a theme function to allow users to change
        // it's general look
        $output = theme('slideshow_creator', $current_ss, $links, $slides, $js);
      }
      break;
  }
  return $output;
}

/**
 * Repair internal urls using the pathfilter module, if available.
 *
 * @param $url
 *   String, The url to complete (if internal)
 * @return
 *   String, The full url, filtered from internal
 */
function _slideshow_creator_url($url) {
  if (module_exists('pathfilter')) {
    // Wrap url in double quotes, filter and then remove double quotes
    return preg_replace("(^\"|\"$)", '', pathfilter_filter('process', 0, -1, '"'. $url .'"'));
  }
  else {
    // Return URL as-is
    return $url;
  }
}

/**
 * Format the slideshow
 * @ingroup themeable
 */
function theme_slideshow_creator(&$ssid, &$links, $ss, $js) {
  // This will put the pieces into a given order,
  // base on which "layout" the user chose.
  if (empty($ss['layout']) or $ss['layout'] == 'default') {
    $layout = array('previous', 'status', 'next', array('title', 'main', 'description'));
  }
  elseif ($ss['layout'] == 'reverse') {
    $layout = array(array('title', 'main', 'description'), 'previous', 'status', 'next');
  }
  elseif ($ss['layout'] == 'bottom') {
    $layout = array('previous', 'status', 'next', array('title', 'description', 'main'));
  }
  elseif ($ss['layout'] == 'top') {
    $layout = array(array('main', 'title', 'description'), 'previous', 'status', 'next');
  }

  // Building the slideshow parts: title, name, slideshow weight and height
  $output_css      = !empty($ss['name']) ? $ss['name'] : '';

  // Setting the image height and width
  $ssc['width']  = (empty($ss['width']) ?  $width  : $ss['width']) .'px';
  $ssc['height'] = (empty($ss['height']) ? $height : $ss['height']) .'px';

  // Create each image. The order should be:
  // 1* the current picture to the end
  // 2* the first picture to the current
  for ($i = $ss['current']; $i <= $ss['total']; ++$i) {
    _scc_content($ssc, $ss, $i);
  }
  for ($i = 1; $i < $ss['current']; ++$i) {
    _scc_content($ssc, $ss, $i);
  }

  // Put a PREVIOUS and NEXT buttons if there are more
  // then one image (generally TRUE for a slideshow)
  if ($ss['total'] > 1) {
    $ssc['previous'] = l(t('Previous'), $_GET['q'], array('query' =>  'ssc='. $links['previous'],
      'attributes' => array('id' => "ssc-previous-$ssid", 'class' => 'ssc-previous')));
    $ssc['next']     = l(t('Next'), $_GET['q'],  array('query' => 'ssc='. $links['next'],
      'attributes' => array('id' => "ssc-next-$ssid", 'class' => 'ssc-next')));
    $ssc['status']   = " <span class='ssc-index' id='ssc-index-$ssid'>".
      t('Image') ." <span class='ssc-current' id='ssc-current-$ssid'>".
      $ss['current'] .'</span>/'. $ss['total'] .' </span>';
  }

  // Put all pieces togheter, according to layout
  foreach ($layout as $piece) {
    if (is_array($piece)) {
      $output .= "<span id='ssc-effect-$ssid' class='ssc-effect'>\n";
      foreach (array_keys($ssc['content']['main']) as $slide) {
        $output .= "<span>\n";
        foreach ($piece as $piece2) {
          $output .= $ssc['content'][$piece2][$slide];
        }
        $output .= "</span>\n";
      }
      $output .= "</span>\n";
    }
    else {
      $output .= $ssc[$piece];
    }
  }

  // Add the CSS file
  drupal_add_css(drupal_get_path('module', 'slideshow_creator') .'/slideshow_creator.css');

  // Add the main JavaScript, that does all the magic
  drupal_add_js(drupal_get_path('module', 'slideshow_creator') .'/slideshow_creator.js');
  drupal_add_js(drupal_get_path('module', 'jquery_plugin') .'/jquery.cycle.min.js');

  // Add specific data for this slideshow
  drupal_add_js(array('ssc' => array($ss['current'] => $js)), 'setting');

  // Put everything inside a SPAN tag
  return  "<span class=\"ssc $output_css\" id=\"ssc-$ssid\">\n$output</span>\n";
}

function _scc_content(&$ssc, &$ss, $i) {
  $title = !empty($ss[$i]['title']) ? $ss[$i]['title'] : '';
  $ssc['content']['title'][] = "<span class='ssc-title' id='ssc-title-$ssid'>$title</span>\n";

  $ssc['content']['description'][] = '<span class="ssc-description" id="ssc-description-'.
      $ssid .'">'. $ss[$i]['description'] ."</span>\n";

  // Build the image
  $image = theme('image', $ss[$i]['src'], check_plain($title),
    NULL, NULL, FALSE);

  // Unless the height and width was deternimed, we must
  // extract these values from each and every image and
  // select the biggest value to fix the image container
  // to this size
  if (empty($ss['width']) and empty($ss['height'])) {
    list($cur_width, $cur_height) = @getimagesize($ss[$i]['src']);
    $ssc['width']  = $cur_width > $width   ? $cur_width  : $width;
    $ssc['height'] = $cur_height > $height ? $cur_height : $height;
  }

  // Add a new image link
  if (empty($ss[$i]['link'])) {
    $ssc['content']['main'][] = $image ."\n";
  }
  else {
    $ssc['content']['main'][] = l($image, $ss[$i]['link'], array('html' => TRUE)) ."\n";
  }
}